/* Vido.cpp : Gestion du club vido (p.223  254)
 *             Les fonctions ont t mises dans l'ordre ncessaire.
 * Auteur    : M.M.
 * Cration  : juillet 1998
 */
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <stdcpp>
using namespace std;

const int NO_FILM_MIN= 1000;       // Premier numro de film possible dans le club
const int NO_FILM_MAX= 9999;       // Dernier numro de film possible dans le club
const int NB_MAX_FILMS= NO_FILM_MAX-NO_FILM_MIN+1;    // Nombre maximum de films
const int NO_ABONNE_MIN= 1;        // Ils seront numrots  partir de cette valeur
const int LARGEUR_TITRE_FILM= 30;  // Largeur maximum des titres de films
const int LARGEUR_NOM_ABONNE= 40;  // Largeur maximum du nom (y compris le prnom)
const int LARGEUR_ADRESSE= 60;     // Largeur maximum des adresses civiques
const int LARGEUR_TELEPHONE= 20;   // Largeur maximum des numros de tlphone
const int LARGEUR_DATE_LOCATION= 10;    // On s'attend au format jj/mm/aaaa
const int NB_LIGNES_ECRAN= 25;    // Nombre de lignes de l'cran

// Noms physiques des fichiers 
const char FICHIER_FILMS[]= "Films.donnes";
const char FICHIER_ABONNES[]= "Abonns.donnes";
const char FICHIER_LOCATIONS[]= "Locations.donnes";

/*****
 * Fonctions pour fichier des films du club vido
 * Champs : 
 * noFilm    : int     entre NO_FILM_MIN et NO_FILM_MAX
 * titre     : string  complet ou simplifi pour ne pas tre trop long (non valid)
 * categorie : char    code reprsentant la catgorie (pour la liste des codes et
 *                     leur signification, voir la fonction NomCompletDeCategorie)
 * duree     : int     en minutes (valid >= 1)
 */
bool Lire(istream& p_ficFilms, 
          int& p_s_noFilm, string& p_s_titre, char& p_s_categorie, int& p_s_duree)
    {
    p_ficFilms >> p_s_noFilm;
    p_ficFilms >> ws; getline(p_ficFilms, p_s_titre, '\t');
    p_ficFilms >> p_s_categorie >> p_s_duree;

    return p_ficFilms.good();
    }

void Ecrire(ostream& p_ficFilms, 
            int p_noFilm, string p_titre, char p_categorie, int p_duree)
    {
    p_ficFilms << p_noFilm << '\t' << p_titre << '\t' << p_categorie << '\t' 
               << p_duree << endl;
    }

/*****
 * Fonctions pour fichier des abonns
 * Champs :
 * noAbonne  : int        allou squentiellement dans le fichier
 * nom       : string     nom (+ prnom) de l'abonn (non valid)
 * adresse   : string     adresse complte (non valid)
 * telephone : string     no de tlphone (non valid)
 */
bool Lire(istream& p_ficAbonnes, 
          int& p_s_noAbonne, string& p_s_nom, string& p_s_adresse, 
          string& p_s_telephone)
    {
    p_ficAbonnes >> p_s_noAbonne;
    p_ficAbonnes >> ws; getline(p_ficAbonnes, p_s_nom, '\t');
    getline(p_ficAbonnes, p_s_adresse, '\t');
    getline(p_ficAbonnes, p_s_telephone);

    return p_ficAbonnes.good();
    }

void Ecrire(ostream& p_ficAbonnes, 
            int p_noAbonne, string p_nom, string p_adresse, string p_telephone)
    {
    p_ficAbonnes << p_noAbonne << '\t' << p_nom << '\t' << p_adresse << '\t' 
                 << p_telephone << endl;
    }

/*****
 * Fonctions pour fichier des locations de films
 * Champs :
 * noAbonne     : int         abonn qui loue le film
 * noFilm       : int         numro du film lou
 * dateLocation : string      date de la location (non valid)
 * prixLocation : double      prix de la location selon type de film
 */
bool Lire(istream& p_ficLocations, 
          int& p_s_noAbonne, int& p_s_noFilm, string& p_s_dateLocation,
          double& p_s_prixLocation)
    {
    p_ficLocations >> p_s_noAbonne >> p_s_noFilm;
    p_ficLocations >> ws; getline(p_ficLocations, p_s_dateLocation, '\t');
    p_ficLocations >> p_s_prixLocation;

    return p_ficLocations.good();
    }

void Ecrire(ostream& p_ficLocations, 
            int p_noAbonne, int p_noFilm, string p_dateLocation,
            double p_prixLocation)
    {
    p_ficLocations << p_noAbonne << '\t' << p_noFilm << '\t' 
                   << p_dateLocation << '\t' << p_prixLocation << endl;
    }

/*****
 * CreerFichiers : Cre les fichiers de films, d'abonns et de locations
 *                 s'ils n'existent pas.
 */
void CreerFichiers()
    {
    ofstream ficFilms(FICHIER_FILMS, ios_base::app);
    ofstream ficAbonnes(FICHIER_ABONNES, ios_base::app);
    ofstream ficLocations(FICHIER_LOCATIONS, ios_base::app);
    }

/*****
 * FairePause : Permet  l'usager de lire les messages qui sont  l'cran.
 */
void FairePause()
    {
    cout << "Tapez la touche Entre pour continuer...";
    cin.sync();
    cin.get();
    cin.sync();
    }

/*****
 * Fonctions non montres dans le livre :
 * (versions simplifies)
 */
void AfficherMenuPrincipal()
    {
    cout << "\n       CLUB VIDO INC.\n\n"
            "1. Ajouter un nouveau film\n"
            "2. Inscrire un nouvel abonn\n"
            "3. Produire la liste des films\n"
            "4. Produire la liste des abonns\n"
            "5. Faire la location d'un film\n"
            "6. Afficher le film le plus populaire (le plus lou)\n"
            "7. Afficher le nombre de locations de tous les films\n\n"
            "0. FIN DU PROGRAMME\n";
    }

char ObtenirChoix(char p_choixMin, char p_choixMax)
    {
	char choix;

	for(;;)
		{
		cout << "Entrez votre choix (" << p_choixMin << '-' << p_choixMax << ") :";
		cin >> choix;
		/***/
		if (p_choixMin <= choix && choix <= p_choixMax) break;
		/***/
		cout << "Vous devez taper un caractre entre " << p_choixMin << " et "
			 << p_choixMax << ".\n"
			 << "Entrez votre choix attentivement : ";
		}

	return choix;
	}

int ObtenirNoFilm(string p_question)
	{
	int noFilm;

	for(;;)
		{
		cout << p_question;
		cin >> noFilm;
		/***/
		if (NO_FILM_MIN <= noFilm && noFilm <= NO_FILM_MAX) break;
		/***/
		cout << "Le numro doit tre entre " << NO_FILM_MIN 
			 << " et " << NO_FILM_MAX << ".\n";
		}

	return noFilm;
	}

char ObtenirCategorieFilm()
    {
	char categorie;

	for(;;)
		{
		cout << "Entrez la catgorie du film (H)orreur, (S)cience-fiction,\n"
                "(C)omdie, (E)nfant, (D)rame ou (R)pertoire : ";
        cin >> categorie;
		/***/
		if (categorie == 'H' || categorie == 'S' || categorie == 'C' 
            || categorie == 'E' || categorie =='D' || categorie == 'R') break;
		/***/
        cout << "Vous devez taper, en majuscules, une des lettres crites entre\n"
                "parenthses. Par exemple, simplement H pour les films d'horreur.\n";
		}

	return categorie;
	}

int ObtenirDureeFilm()
    {
	int duree;

	for(;;)
		{
		cout << "Entrez la dure de ce film, en minutes : ";
        cin >> duree;
		/***/
		if (duree >= 1) break;
		/***/
		cout << "La dure doit tre donne en minutes (au moins une minute).\n";
		}

	return duree;
	}

string NomCompletDeCategorie(char p_codeCategorie)
    {
	switch (p_codeCategorie)
		{
		case 'H' : return "Horreur";
		case 'S' : return "Science-fiction";
		case 'C' : return "Comdie";
		case 'E' : return "Enfant";
		case 'D' : return "Drame";
		case 'R' : return "Rpertoire";
		}

	return "Erreur";
	}

/*****
 * FilmExiste : Vrifie si le film est inscrit dans le fichier des films.
 */
bool FilmExiste(int p_noFilmCherche)
    {
    ifstream ficFilms(FICHIER_FILMS);    // Ses champs suivent
    int noFilm;
    string titre;
    char categorie;
    int duree;
 
    while (Lire(ficFilms, noFilm, titre, categorie, duree))
        {
        if (noFilm == p_noFilmCherche)
            return true;
        }
 
    return false;
    }
 
/*****
 * AjouterFilm : Ajoute un nouveau film dans le fichier des films
 */
void AjouterFilm()
    {
    int noFilm= ObtenirNoFilm("Entrez le numro du nouveau film : ");

    if (FilmExiste(noFilm))
        cout << "Ce film existe dj dans le club vido.\n";
    else
        {
        cout << "Entrez le titre du nouveau film : ";
        string titre;                                  // Champ du fichier de films
        cin >> ws; getline(cin, titre);

        char categorie= ObtenirCategorieFilm();
        int duree= ObtenirDureeFilm();

        ofstream ficFilms(FICHIER_FILMS, ios_base::app);
        Ecrire(ficFilms, noFilm, titre, categorie, duree);
        }
    }
 
/*****
 * InscrireAbonne : Ajoute un nouvel abonn dans le fichier des abonns.
 */
void InscrireAbonne()
    {
    int cptAbonnes= 0;    // Comptera le nombre d'abonns dj inscrits

    fstream ficAbonnes(FICHIER_ABONNES, ios_base::in);    // Ses champs suivent
    int noAbonne;
    string nom;
    string adresse;
    string telephone;

    while (Lire(ficAbonnes, noAbonne, nom, adresse, telephone))
        ++cptAbonnes;
 
    ficAbonnes.close();
    ficAbonnes.clear();	// Pour annuler eof()
 
    noAbonne= NO_ABONNE_MIN + cptAbonnes;
    cout << "Cet abonn portera le numro " << noAbonne << ".\n";
 
    cout << "Quel est son nom ? ";
    cin >> ws; getline(cin, nom);
    cout << "Quel est son adresse ? ";
    getline(cin, adresse);
    cout << "Quel est son numro de tlphone ? ";
    getline(cin, telephone);
     
    ficAbonnes.open(FICHIER_ABONNES, ios_base::out|ios_base::app);
    Ecrire(ficAbonnes, noAbonne, nom, adresse, telephone);
    ficAbonnes.close();
 
    FairePause();
    }
 
/*****
 * ProduireListeFilms : Affiche la liste de tous les films, avec pause et
 *     formatage. Donne aussi le nombre total de films (sommaire).
 */
void ProduireListeFilms()
    {
    ifstream ficFilms(FICHIER_FILMS);    // Ses champs suivent
    int noFilm;
    string titre;
    char categorie;
    int duree;

    // Le numro de ligne sera entre 0 et NB_LIGNES_ECRAN-1 pour indiquer  quelle
    // ligne on est rendu  crire  l'cran. Au dpart, il dpasse la limite pour
    // forcer l'affichage de l'en-tte.
    int noLigne= NB_LIGNES_ECRAN; 

    int cptFilms= 0;

    while (Lire(ficFilms, noFilm, titre, categorie, duree))
        {
        // On vrifie que l'criture qui suit ne placerait pas la prochaine
        // hors de l'cran.
        if (noLigne+1 >= NB_LIGNES_ECRAN) 
            {
            if (cptFilms > 0) FairePause();    // Pas de pause au dpart

            cout << "Numro  " << left << setw(LARGEUR_TITRE_FILM) << "Titre"
                 << "  Catgorie  Dure\n"
                 << "------  "
                 << setfill('-') << setw(LARGEUR_TITRE_FILM) << '-' << setfill(' ')
                 << "  ---------  -----\n";
            noLigne= 2;
            }

        cout << right << setw(6) << noFilm << "  " 
             << left << setw(LARGEUR_TITRE_FILM) << titre << "  "
             << NomCompletDeCategorie(categorie) << "  "
             << right << setw(5) << duree << endl;

        ++noLigne;
        ++cptFilms;
        }

    ficFilms.close();

    if (cptFilms == 0)
        cout << "Il n'y aucun film inscrit.\n";
    else
        {
        // On a trois lignes  crire dans le sommaire, qu'on veut laisser ensemble,
        // le test pour pause est donc :
        if (noLigne+3 >= NB_LIGNES_ECRAN) FairePause();

        cout << "------------------------\n"
             << "Nombre de films : " << cptFilms << endl
             << "------------------------\n";
        }

    // Toujours une pause avant le retour au menu, pour laisser le temps de lire
    // les renseignements du rapport.
    FairePause();
    }

/*****
 * ProduireListeAbonnes : Affiche la liste de tous les abonns, avec pause et
 *     et formatage. L'affichage d'un abonn est sur trois lignes...
 */
void ProduireListeAbonnes()
    {
    ifstream ficAbonnes(FICHIER_ABONNES);    // Ses champs suivent
    int noAbonne;
    string nom;
    string adresse;
    string telephone;

    // Le numro de ligne sera entre 0 et NB_LIGNES_ECRAN-1 pour indiquer  quelle
    // ligne on est rendu  crire  l'cran. Au dpart, il dpasse la limite pour
    // forcer l'affichage de l'en-tte.
    int noLigne= NB_LIGNES_ECRAN; 

    int cptAbonnes= 0;

    while (Lire(ficAbonnes, noAbonne, nom, adresse, telephone))
        {
        // On vrifie que l'criture qui suit ne placerait pas la prochaine
        // hors de l'cran.
        if (noLigne+3 >= NB_LIGNES_ECRAN) 
            {
            if (cptAbonnes > 0) FairePause(); // Pas de pause au dpart

            noLigne= 0;
            }

        cout << "Abonn " << noAbonne << ", " << nom << endl
             << "    Adresse   : " << adresse << endl
             << "    Tlphone : " << telephone << endl;
             
        noLigne+= 3;
        ++cptAbonnes;

        // On veut une ligne blanche entre chaque abonn ou aprs la pause :
        if (noLigne+1 >= NB_LIGNES_ECRAN) 
            {
            FairePause();
            noLigne= 0;
            }

        cout << endl;
        ++noLigne;
        }

    ficAbonnes.close();

    if (cptAbonnes == 0)
        cout << "Il n'y aucun abonne.\n";

    // Laisser le temps de lire les renseignements du rapport.
    FairePause();
    } 

/*****
 * AbonneExiste : Vrifie si l'abonn est inscrit dans le fichier des abonns.
 */
bool AbonneExiste(int p_noAbonneCherche)
    {
    ifstream ficAbonnes(FICHIER_ABONNES);    // Ses champs suivent
    int noAbonne;
    string nom;
    string adresse;
    string telephone;

    while (Lire(ficAbonnes, noAbonne, nom, adresse, telephone))
        {
        if (noAbonne == p_noAbonneCherche)
            return true;
        }

    return false;
    }

/*****
 * DeterminerPrixLocation : Calcule le prix de la location d'aprs le numro
 *     du film. Le prix dpend de la catgorie qui est retrouve dans le fichier
 *     des films.
 */
double DeterminerPrixLocation(int p_noFilm)
	{
	ifstream ficFilms(FICHIER_FILMS);    // Ses champs suivent
	int noFilm;
	string titre;
	char categorie;
	int duree;
	 
	while (Lire(ficFilms, noFilm, titre, categorie, duree)
	       && noFilm != p_noFilm)
		; // rien d'autres  faire
	 
    double prix; // Prix de la location selon catgorie
	 
    switch (categorie)
		{
		case 'E' : prix= 2.50; break;
		case 'H' : prix= 3.50; break;
		case 'R' : prix= 3.00; break;
		default  : prix= 4.00; break;
		}
	 
	return prix;
	}

/*****
 * LouerFilm : Ajoute une location de film dans le fichier des locations.
 */
void LouerFilm()
    {
    cout << "Entrez le numro de l'abonn dsirant lou un film : ";
    int noAbonne;    // Champ des fichiers d'abonns et locations
    cin >> noAbonne;

    if (!AbonneExiste(noAbonne))
        {
        cout << "Dsol, ce numro d'abonn n'est pas valide dans ce club vido.\n";
        FairePause();
        return;
        }

    int noFilm;    // Champ des fichiers films et locations

    for(;;)
        {
        noFilm= ObtenirNoFilm("Entrez le numro du film  louer : ");
        /***/
        if (FilmExiste(noFilm)) break;
        /***/
        cout << "Ce numro de film ne correspond  aucun des films du club.\n";
        }

    cout << "Entrez la date de location de ce film : ";
    string dateLocation;    // Champ du fichier de locations
    cin >> dateLocation;

    double prixLocation= DeterminerPrixLocation(noFilm);
    cout << "Cette location cotera " 
         << fixed << setprecision(2) << prixLocation << " $.\n";

    ofstream ficLocations(FICHIER_LOCATIONS, ios_base::app);
    Ecrire(ficLocations, noAbonne, noFilm, dateLocation, prixLocation);
    ficLocations.close();

    FairePause();
    }

/*****
 * CumulerNbLocations : remplit le vecteur des nombres de locations et renvoie
 *        aussi le nombre total de locations.
 */
long CumulerNbLocations(int p_s_tabLocations[NB_MAX_FILMS])
    {
    for (int i= 0; i < NB_MAX_FILMS; ++i)
        p_s_tabLocations[i]= 0;

    ifstream ficLocations(FICHIER_LOCATIONS);    // Ses champs suivent
    int noAbonne;
    int noFilm;
    string dateLocation;
    double prixLocation;

    while (Lire(ficLocations, noAbonne, noFilm, dateLocation, prixLocation))
       ++p_s_tabLocations[noFilm-NO_FILM_MIN];

    int cumulLocations= 0;

    for (int i= 0; i < NB_MAX_FILMS; ++i)
        cumulLocations+= p_s_tabLocations[i];

    return cumulLocations;
    }

/*****
 * AfficherFilmPlusPopulaire : Affiche toutes les informations sur le film qui a
 *     t lou le plus de fois. N'en montre qu'un, mme si plusieurs autres films
 *     ont autant de locations...
 */
void AfficherFilmPlusPopulaire()
    {
    int tabLocations[NB_MAX_FILMS];    // Le nombre de locations de chaque film
    
    if (CumulerNbLocations(tabLocations) == 0)
        cout << "Aucun film n'a t lou jusqu' maintenant.\n";
    else
        {
        int indiceFilmPlusLoue= 0;   // On en retiendra qu'un...
        int cptFilmsPlusLoues= 1;    // Mais on va tous les compter

        for (int i= 1; i < NB_MAX_FILMS; ++i)
            {
            if (tabLocations[i] > tabLocations[indiceFilmPlusLoue])
                {
                indiceFilmPlusLoue= i;
                cptFilmsPlusLoues= 1;
                }
            else
                {
                if (tabLocations[i] == tabLocations[indiceFilmPlusLoue])
                    ++cptFilmsPlusLoues;
                }
            }

        ifstream ficFilms(FICHIER_FILMS);    // Ses champs suivent
        int noFilm;
        string titre;
        char categorie;
        int duree;
 
        while (Lire(ficFilms, noFilm, titre, categorie, duree)
               && noFilm != indiceFilmPlusLoue+NO_FILM_MIN)
            ; // rien d'autres  faire
 
        if (cptFilmsPlusLoues == 1)
            {
            cout << "Le film le plus populaire est " << titre << endl
                 << "qui porte le numro " << noFilm 
                 << " et a t lou " << tabLocations[indiceFilmPlusLoue] 
                 << " fois.\n"
                 << "Dure : " << duree << " minutes, catgorie " 
                 << NomCompletDeCategorie(categorie) << ".\n";
            }
        else
            {
            cout << "Il y a " << cptFilmsPlusLoues 
                 << " films qui se partagent la palme du film le plus populaire\n"
                 << "parce qu'ils ont t lous " 
                 << tabLocations[indiceFilmPlusLoue] << " fois.\n"
                 << "L'un d'entre eux est " << titre << endl
                 << "qui porte le numro " << noFilm << ", dure " << duree
                 << " minutes, catgorie " << NomCompletDeCategorie(categorie)
                 << ".\n";
            }
       }

    FairePause();
    }

/*****
 * AfficherListeLocations : Affiche le nombre de locations des films lous au moins
 *                          une fois et le nombre total de locations.
 */
void AfficherListeLocations()
    {
    int tabLocations[NB_MAX_FILMS];    // Le nombre de locations de chaque film
    
    int nbTotalLocations= CumulerNbLocations(tabLocations);

    if (nbTotalLocations == 0)
        cout << "Aucun film n'a t lou jusqu' maintenant.\n";
    else
        {
        ifstream ficFilms(FICHIER_FILMS);    // Ses champs suivent...
        int noFilm;
        string titre;
        char categorie;
        int duree;

        int noLigne= NB_LIGNES_ECRAN;    // Forcera en-tte initial

        while (Lire(ficFilms, noFilm, titre, categorie, duree))
            {
            if (tabLocations[noFilm-NO_FILM_MIN] != 0)
                {
                if (noLigne+1 >= NB_LIGNES_ECRAN)
                    {
                    // Pause sauf si premire fois
                    if (noLigne != NB_LIGNES_ECRAN) FairePause();

                    //       123456**123456789...(utile pour largeurs dans setw)
                    cout << "Numro   Nombre\n"
                         << "  du       de\n"
                         << " film   locations  Titre\n"
                         << "------  ---------  " 
                         << setfill('-') << setw(LARGEUR_TITRE_FILM) << '-'
                         << setfill(' ') << endl;
                    noLigne= 4;
                    }

                cout << right << setw(6) << noFilm << "  "
                     << setw(9) << tabLocations[noFilm-NO_FILM_MIN] << "  "
                     << left << titre << endl;
                ++noLigne;
                }
            }

        if (noLigne+2 >= NB_LIGNES_ECRAN) FairePause();
        cout << "--------------------------------------\n"
             << "Nombre total de locations : " << nbTotalLocations << endl;
        }

    FairePause();
    }

/*****
 * PROGRAMME PRINCIPAL
 */
int main()
    {
    CreerFichiers();

    for(;;)
        {
        AfficherMenuPrincipal();
        char choix= ObtenirChoix('0','7');
        /***/
        if (choix == '0') break;
        /***/
        switch (choix)
            {
            case '1' : AjouterFilm(); break;
            case '2' : InscrireAbonne(); break;
            case '3' : ProduireListeFilms(); break;
            case '4' : ProduireListeAbonnes(); break;
            case '5' : LouerFilm(); break;
            case '6' : AfficherFilmPlusPopulaire(); break;
            default  : AfficherListeLocations(); break;
            }
        }
    }
